
# This file was generated by 'versioneer.py' (0.7+) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = 'v0.2'
version_full = '2895ea7e0bcf6b16c5e5bc98a10fd46d8e05e172'
def get_versions(default={}, verbose=False):
    return {'version': version_version, 'full': version_full}

